﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Windows.Controls.Ribbon;
using JasRibbonApplicatie.Views;
using JasRibbonApplicatie.Classes;
using JasRibbonApplicatie.Views.BeheerAanvragen;

namespace JasRibbonApplicatie
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : RibbonWindow
	{
        // geen constanten -- waarom hoofdletters?
		private LinearGradientBrush SELECTED_BGCOLOR = new LinearGradientBrush();
		private Brush SELECTED_BORDER_COLOR = Brushes.Orange;
		private Login login = null;
		public Page currentPage;
		public Aanvraag SelectedAanvraag { private set; get; }

		public MainWindow()
		{
			InitializeComponent();
			
			#region init selection brush
			SELECTED_BGCOLOR.GradientStops.Add(new GradientStop(Color.FromRgb(255, 253, 249), 0.0));
			SELECTED_BGCOLOR.GradientStops.Add(new GradientStop(Color.FromRgb(253, 222, 185), 0.39));
			SELECTED_BGCOLOR.GradientStops.Add(new GradientStop(Color.FromRgb(255, 207, 111), 0.40));
			SELECTED_BGCOLOR.GradientStops.Add(new GradientStop(Color.FromRgb(255, 234, 168), 1.0));
			SELECTED_BGCOLOR.Opacity = 60;
			SELECTED_BGCOLOR.StartPoint = new Point(0, 0);
			SELECTED_BGCOLOR.EndPoint = new Point(0, 1);
			#endregion

			login = Login.Instance;
			login.MainWindowReference = this;
			Aanmelden();
			//onLoadCompleted => methode uitvoeren die content set in currentPage dit om vanaf de ribon een page te kunnen "besturen"
			mainFrame.NavigationService.LoadCompleted += new System.Windows.Navigation.LoadCompletedEventHandler(NavigationService_LoadCompleted);
		}
		
		// Summary:
		//      Closes the entire program
		//
		private void Close_Click(object sender, RoutedEventArgs e)
		{
			this.Close();
		}

		// Summary:
		//      Closes the main application and opens the login window, so that a new user can login.
		//
		private void Logoff_Click(object sender, RoutedEventArgs e)
		{
			login.UserData.save();
			login.userLogOff();
			Aanmelden();
		}

		public void Aanmelden()
		{
			if (login.isLoggedIn)
			{
                scannenTab.IsSelected = true;
                //scannenTab.RaiseEvent(new RoutedEventArgs(;
                materiaalTab.Visibility = Visibility.Hidden;
                aanvragenTab.Visibility = Visibility.Hidden;
                gebruikersTab.Visibility = Visibility.Hidden;
                planningTab.Visibility = Visibility.Hidden;

                if (login.IsBeheerder)
                {
                    materiaalTab.Visibility = Visibility.Visible;
                    aanvragenTab.Visibility = Visibility.Visible;
                    gebruikersTab.Visibility = Visibility.Visible;
                    planningTab.Visibility = Visibility.Visible;
                    materiaalTab.IsEnabled = true;
                    aanvragenTab.IsEnabled = true;
                    gebruikersTab.IsEnabled = true;
                    planningTab.IsEnabled = true;
                }
				Logoff.IsEnabled = true;
				scannenTab.IsEnabled = true;

				mainFrame.Source = new Uri("Views/Ontleningen.xaml", UriKind.Relative);
				this.Show();
			}
			else
			{
				Logoff.IsEnabled = false;
				scannenTab.IsEnabled = true;
				materiaalTab.IsEnabled = true;
				aanvragenTab.IsEnabled = false;
				gebruikersTab.IsEnabled = false;
				planningTab.IsEnabled = false;
				this.Hide();
				AanmeldWindow aanmeldWindow = new AanmeldWindow(this);
				aanmeldWindow.ShowDialog();
			}
		}

		void NavigationService_LoadCompleted(object sender, System.Windows.Navigation.NavigationEventArgs e)
		{
			currentPage = ((Page)e.Content);
		}
		
		private void onButton_Click(object sender, RoutedEventArgs e)
		{
			String buttonName = ((Button)(sender)).Name;
			String uri = string.Empty;

			switch (buttonName)
			{
				case "btnNewItem":
					uri = "Views/NieuwItemBarcodeToewijzen.xaml";
					break;
				case "btnRefreshItem":
					uri = "Views/BarcodeVernieuwen.xaml";
					break;
				case "btnPrintLabels":
					uri = "Views/PrintLabels.xaml";
					break;
				case "gebruikersLijstButton":
					uri = "Views/BeheerGebruikers.xaml";
					break;
				case "planningButton":
					uri = "Views/MagazijniersPlanning.xaml";
					break;
				default:
					uri = "Views/Home.xaml";
					break;
			}
			mainFrame.Source = new Uri(uri, UriKind.Relative);
		}

        // nodig voor ontleningen (default button aanzetten)
        public LinearGradientBrush GetButtonBrush()
        {
            return SELECTED_BGCOLOR;
        }

		private void onOntleningButton_Click(object sender, RoutedEventArgs e)
		{
			Button button = ((Button)sender);

			var huidigePagina = currentPage as Ontleningen;
            int status = 0;
			switch (button.Name)
			{
				case "btnOntlenen":
                    status = Ontleningen.STATUS_ONTLENING;
					break;
				case "btnTerugbrengen":
                    status = Ontleningen.STATUS_TERUGBRENGEN;
					break;
				default:
                    status = Ontleningen.STATUS_ONTLENING;
					break;
			}            

            huidigePagina.ChangeStatus(status); // zodat de functionaliteit aangepast wordt...

            // visuele reactie op wat geselecteerd is...
            if (status == Ontleningen.STATUS_ONTLENING)
            {
                this.btnTerugbrengen.Background = null;
                this.btnTerugbrengen.BorderBrush = null;
                this.btnOntlenen.Background = SELECTED_BGCOLOR;
                this.btnOntlenen.BorderBrush = Brushes.Crimson;
                
            }
            else
            {
                this.btnOntlenen.Background = null;
                this.btnOntlenen.BorderBrush = null;
                this.btnTerugbrengen.Background = SELECTED_BGCOLOR;
                this.btnTerugbrengen.BorderBrush = Brushes.Crimson;
            }
		}

		private void overzichtMateriaalStatus_Click(object sender, RoutedEventArgs e)
		{
			Button btn = (Button)(sender);
			BeheerMateriaal bm = currentPage as BeheerMateriaal;
            if(bm!=null) bm.ButtonChoice(btn.Name);
            else mainFrame.Source = new Uri("Views/BeheerMateriaal.xaml", UriKind.Relative);
		}

		private void aanvraagFilterStatus_Click(object sender, RoutedEventArgs e)
		{
			Button btn = (Button)(sender);
			
			var ba = currentPage as OverzichtAanvragen;
			Aanvraag.Statussen? status;
			switch (btn.Name)
			{
				case "nieuweAanvragenButton":
					status = Aanvraag.Statussen.nieuw;
					break;
				case "goedgekeurdeAanvragenButton":
					status = Aanvraag.Statussen.goedgekeurd;
					break;
				case "behandeldeAanvragenButton":
					status = Aanvraag.Statussen.behandeld;
					break;
				case "verstrekenAanvragenButton":
					status = Aanvraag.Statussen.verstreken;
					break;
				case "afgekeurdeAanvragenButton":
					status = Aanvraag.Statussen.afgekeurd;
					break;
				default:
					status = null;
					break;
			}
			if (btn.Background != SELECTED_BGCOLOR)
			{
				ba.addToStatusFilter(status.Value);
				btn.Background = SELECTED_BGCOLOR;
				btn.BorderBrush = SELECTED_BORDER_COLOR;
			}
			else
			{
				ba.RemoveFromStatusFilter(status.Value);
				btn.Background = null;
				btn.BorderBrush = null;
			}
		}

		private void onTab_Click(object sender, SelectionChangedEventArgs e)
		{
			Ribbon r = (Ribbon)sender;
			RibbonTab rt = (RibbonTab)r.SelectedItem;
			String uri = string.Empty;
            
			switch (rt.Name)
			{
				case "scannenTab":
                    uri = "Views/Ontleningen.xaml";
					break;
				case "materiaalTab":
					uri = "Views/BeheerMateriaal.xaml";                  
					break;
				case "aanvragenTab":
					uri = "Views/BeheerAanvragen/OverzichtAanvragen.xaml";
					break;
				case "GebruikersTab":
					uri = "Views/BeheerGebruikers.xaml";
					break;
				case "PlanningTab":
					uri = "Views/MagazijniersPlanning.xaml";
					break;
			}
			if (uri != string.Empty)
				mainFrame.Source = new Uri(uri, UriKind.Relative);

		}

		private void AanvragenTools_Click(object sender, RoutedEventArgs e)
		{
			Button btn = (Button)(sender);
			string uri = string.Empty;

			switch (btn.Name)
			{
				case "AanvraagKiezenButton":
					if (currentPage.GetType() != typeof(OverzichtAanvragen)){
						SelectedAanvraag = null;
						uri = "Views/BeheerAanvragen/OverzichtAanvragen.xaml";
					}
					break;
				case "AanvraagMateriaalButton":
					if (currentPage.GetType() == typeof(OverzichtAanvragen))
					{
						OverzichtAanvragen p = currentPage as OverzichtAanvragen;
						SelectedAanvraag = p.selectedAanvraag;
					}
					if (SelectedAanvraag != null) uri = "Views/BeheerAanvragen/MateriaalAanvragen.xaml";
					break;
				case "AanvraagMailsButton":
					if (currentPage.GetType() == typeof(OverzichtAanvragen))
					{
						OverzichtAanvragen p = currentPage as OverzichtAanvragen;
						SelectedAanvraag = p.selectedAanvraag;
					}
					if (SelectedAanvraag != null) uri = "Views/BeheerAanvragen/MailsAanvragen.xaml";
					break;
				case "AanvraagFormulierenButton":
					if (currentPage.GetType() == typeof(OverzichtAanvragen))
					{
						OverzichtAanvragen p = currentPage as OverzichtAanvragen;
						SelectedAanvraag = p.selectedAanvraag;
					}
					if (SelectedAanvraag != null) uri = "Views/BeheerAanvragen/FormulierenAanvragen.xaml";
					break;
			}
			if (uri != string.Empty)
				mainFrame.Source = new Uri(uri, UriKind.Relative);
		}

		private void btnScanActie_Click(object sender, RoutedEventArgs e)
		{
            var huidigePagina = currentPage as Ontleningen;
            Button button = (Button)sender;
            if (button.Name.Equals("btnOntleningBeheer"))
            {
                huidigePagina.ontleningBeherenButton_Click();
            }
            else if (button.Name.Equals("btnHuidigeAanvragen"))
            {
                huidigePagina.AanvragenVandaagButtonClick();
            }
            else if (button.Name.Equals("btnOngendaanMaken"))
            {
                huidigePagina.undo();
            }
            // nog undo implementeren
		}
	}
}
